package GStandardApplication.ihm;

/**
*
* $Id: GMainFrameAbstract.java 2184 2017-08-08 16:21:03Z jjct $
*
* HISTORIQUE
* VERSION:1.9:DM:DM-614:08/08/2019:[GENIUS] Indication de chargement d'une configuration
* VERSION:1.6:DM:DM-163:31/01/2017: Creation of an standard application widget. 
* VERSION:1.6.2:DM:DM-201:08/08/2017: Configured stdout console to be read only 
*                                      and to scroll always to the bottom. 
* VERSION:1.6.2:FA:FA-203:08/08/2017: Configured the main GFrame to exit on close. 
* VERSION:1.8:FA:FA-73:17/09/2018: Added clearConstraint, clearInnerDescendantConstraint, and
*                                      clearAllInnerDescendantConstraint methods to clear component constraints.
* FIN-HISTORIQUE
*
* This software is part of GENIUS,
* A CNES tool to develop MMI in Java
* Copyright 2016-2026 CNES Centre National d’Etudes Spatiales
* Licensed to Centre National d’Etudes Spatiales (CNES) under one or more
* contributor license agreements. See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* CNES licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License. You may obtain a copy of the License at
*
*   http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*
**/

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.logging.Level;

import javax.swing.JSplitPane;
import javax.swing.UIManager;

import fr.cnes.genius.constraints.GConstraint;
import fr.cnes.genius.constraints.GConstraint.XAlign;
import fr.cnes.genius.exception.GException;
import fr.cnes.genius.exception.GFileManipulatorException;
import fr.cnes.genius.highLevel.GMenu;
import fr.cnes.genius.highLevel.GMenuBar;
import fr.cnes.genius.highLevel.GMenuItem;
import fr.cnes.genius.interval.GIntervalAbstract.GStatus;
import fr.cnes.genius.loggers.GConsoleLogger;
import fr.cnes.genius.lowLevel.GButton;
import fr.cnes.genius.lowLevel.GFrame;
import fr.cnes.genius.lowLevel.GIcon;
import fr.cnes.genius.lowLevel.GLabel;
import fr.cnes.genius.lowLevel.GPanel;
import fr.cnes.genius.lowLevel.GProgressBar;
import fr.cnes.genius.lowLevel.GSplitPane;
import fr.cnes.genius.lowLevel.GStandardDialog.TypeOfDialog;
import fr.cnes.genius.lowLevel.GStandardMessageDialog;
import fr.cnes.genius.lowLevel.GTextArea;
import fr.cnes.genius.main.GEnvironment;
import fr.cnes.genius.main.GEvent;
import fr.cnes.genius.main.GListener;
import fr.cnes.genius.savefiles.GSaveResults;
import fr.cnes.genius.standardapplication.GAboutDialog;
import fr.cnes.genius.standardapplication.GDataPanelAbstract;
import fr.cnes.genius.threads.GCommandLauncher.GExecButton;
import fr.cnes.genius.threads.GCommandLauncher.ProcessStatus;
import fr.cnes.genius.threads.GJavaCommandLauncher;
import fr.cnes.genius.utils.GContextFileManagement;

/**
* Abstract class using as a basis for upper level class defining the top level of the GUI.
* 
* @param <T> Type of the data panel of the application.
* 
* @author goesterjf
* 
*/
public abstract class GMainFrameAbstractNew<T extends GDataPanelAbstract> extends GPanel implements
       GListener {
   /**
    * Standard vertical gap left.
    * 
    */
   private static final int GAP_LEFT = 10;

   /**
    * Standard vertical gap right.
    */
   private static final int GAP_RIGHT = 10;

   /**
    * Debug flag.
    */
   private static boolean sDebugMode = false;

   /**
    * String to indicate debug mode is not active.
    */
   private static String sDebugFalseString;

   /**
    * String to indicate debug mode is active.
    */
   private static String sDebugTrueString;

   /**
    * Load context string, to be used in the main menu and main button bar.
    */
   private static String s_loadContextString;

   /**
    * Save context string, to be used in the main menu and main button bar.
    */
   private static String s_saveContextString;

   /**
    * Save results string, to be used in the main menu and main button bar.
    */
   private static String s_saveResultsString;

   /**
    * Clear messages string, to be used in the main menu and main button bar.
    */
   private static String s_clearMessages;

   /**
    * Clear results string, to be used in the main menu and main button bar.
    */
   private static String s_clearResults;

   /**
    * Quit string, to be used in the main menu and main button bar.
    */
   private static String s_quitString;

   private final GMainFrame m_mainFrame;

   /**
    * Dialog widgets for managing results saving (is accessible via derived classes).
    */
   private final GSaveResults m_saveResultsDialog;

   /**
    * Maib data panel.
    */
   private final T m_dataPanel;

   /**
    * Command launcher (is accessible via derived classes).
    */
   private final GJavaCommandLauncher m_javaCommandLauncher;

   /**
    * Menu bar.
    */
   private GMenuBar m_menuBar;

   /** Menu item for loadin context. */
   private GMenuItem m_menuItemLoadContext;
   /** Menu item for saving context. */
   private GMenuItem m_menuItemSaveContext;
   /** Menu item for saving results. */
   private GMenuItem m_menuItemSaveResults;
   /** Menu item for run. */
   private GMenuItem m_menuItemRun;
   /** Menu item for quit. */
   private GMenuItem m_menuItemQuit;

   /** Menu item for clearing results. */
   private GMenuItem m_menuItemClearResults;
   /** Menu item for clearing messages. */
   private GMenuItem m_menuItemClearMessages;

   /** Menu item for debug mode. */
   private GMenuItem m_menuItemDebugMode;

   /** Menu item for "about". */
   private GMenuItem m_menuItemAbout;

   /** About dialog widget. */
   private final GAboutDialog m_aboutDialog;

   /** Explorer for searching context files. */
   private final GContextFileManagement m_contextFileManagement;

   /** Progress bar. */
   private GProgressBar m_progressBar;

   /** Error console. */
   private final GTextArea m_errorConsole;

   /** Button for loading context. */
   private GButton m_loadContextButton;
   /** Button for saving context. */
   private GButton m_saveContextButton;
   /** Button for saving results. */
   private GButton m_saveResultsButton;
   /** Button for propagating a trajectory. */
   private GExecButton m_runButton;
   /** Button for clearing messages. */
   private GButton m_clearMessagesButton;
   /** Button for clearing results. */
   private GButton m_clearResultsButton;
   /** Button for clearing results. */
   private GButton m_quitButton;

   /** label of the loaded context file. */
   private final GLabel contextLabel;

   /** GEntryString for the name of the loaded context file. */
   private final GLabel contextFileNameLabel;

   /** Name of the loaded context file. */
   private String contextFileName;

   /** Height (in pixels) of the data panel. */
   private int dataPanelHeight;

   /** Height (in pixels) of the error console. */
   private final int errConsoleHeight;
   
   /** GSplit object to separate error console and data panel */
   private final GSplitPane m_split;
   /** GPanel including the textearea console */
   private final GPanel m_errorConsolePan;
   /** GPanel including the data panel */
   private final GPanel m_dataPanelPan;

   /**
    * Main frame class.
    * 
    * @author jjct
    * 
    */
   private static class GMainFrame extends GFrame implements GListener {

       /**
        * Constructor.
        * 
        * @param title frame title.
        * @param panel main panel.
        * @param menubar main menubar.
        */
       public GMainFrame(final String title, GPanel panel, GMenuBar menubar) {
           super(title, panel, menubar);
       }

       /**
        * {@inheritDoc}
        */
       @SuppressWarnings("PMD.UnusedFormalParameter")
       @Override
       public void before(GEvent event) throws GException {
           // Intentionally empty.
       }

       /**
        * {@inheritDoc}
        */
       @Override
       public void after(GEvent event) throws GException {
           if (event.getFinalSource() == this) {
               GStandardMessageDialog quitConfirmDialog = new GStandardMessageDialog("Exit",
                       "Do you really want to quit ?", TypeOfDialog.OK_CANCEL);
               quitConfirmDialog.show();
               boolean isCancelled = quitConfirmDialog.getCancelled();
               quitConfirmDialog.dispose();

               if (isCancelled) {
                   setDefaultCloseOperation(CloseAction.DoNothinOnClose);
               } else {
                   setDefaultCloseOperation(CloseAction.ExitOnClose);
               }
           }

       }

   }

   /**
    * Constructor.
    * 
    * @param title title of the main frame.
    * @param dataPanel panel where all data needed for the appllication are displayed.
    * @param contextFileManagement GFile manager that will be displayed for searching context
    *        files.
    * @param aboutDialog Widget displaying information about the application.
    * @param saveResultsDialog Widget managing the save of the context+results files.
    * @param dataPanelHeight height (in pixels) of the data panel.
    * @param errConsoleHeight height (in pixels) of the error console.
    * @param iconSize size of the icons.
    * @param showProgressBar boolean to decide if a progress bar is displayed.
    */
   public GMainFrameAbstractNew(final String title, final T dataPanel,
           final GContextFileManagement contextFileManagement, final GAboutDialog aboutDialog,
           final GSaveResults saveResultsDialog, final int dataPanelHeight,
           final int errConsoleHeight, final int iconSize, final boolean showProgressBar) {

       super();

       // Intialise the static strings.
       initStrings();

       // Internal attributes initialization
       m_dataPanel = dataPanel;       
       m_contextFileManagement = contextFileManagement;
       m_aboutDialog = aboutDialog;
       m_saveResultsDialog = saveResultsDialog;
       this.errConsoleHeight = errConsoleHeight;
       this.dataPanelHeight = dataPanelHeight;
       // Error console initialization (except redirection of output (see at the end of the code)
       m_errorConsole = new GTextArea();

       // Configure the error console to scroll to the last message.
       m_errorConsole.setEditable(false);
       m_errorConsole.setCursorPositionAtBottom(true);

       GConsoleLogger.iniConsoleLogger(m_errorConsole);

       // Command launcher initialization
       final String[] comm = { null, null };
       m_javaCommandLauncher = new GJavaCommandLauncher(comm, null, "Compute", "Stop compute",
               m_dataPanel.getConsole(), m_errorConsole);
       m_javaCommandLauncher.setButtonIcons(new GIcon(GIcon.Type.START, iconSize), new GIcon(
               GIcon.Type.STOP, iconSize));

       if (showProgressBar) {
           m_progressBar = m_javaCommandLauncher.getGProgressBar();
       }

       createMenuBar();

       createButtonBar(iconSize);

       contextLabel = new GLabel("Loaded context:");

       contextFileName = "no context";
       contextFileNameLabel = new GLabel(contextFileName);
       contextFileNameLabel.getJLabel().setFont(new Font(null, Font.ITALIC, getDefaultFontSize()));

       m_errorConsolePan = new GPanel() {
           @Override
           public void display() throws GException {
               generic();
           }
           @Override
           public void generic() throws GException {
               put(m_errorConsole);
           }
       };

       m_dataPanelPan = new GPanel() {
           @Override
           public void display() throws GException {
               generic();
           }
           @Override
           public void generic() throws GException {
               put(contextLabel);
               put(contextFileNameLabel);
               put(m_dataPanel);
           }
       };       
       
       // Split to adjust the size of the components of the panel
       m_split = new GSplitPane("configSplit", JSplitPane.VERTICAL_SPLIT, m_errorConsolePan, m_dataPanelPan);

       setDefaultConstraints();
       m_mainFrame = new GMainFrame(title, this, m_menuBar);
       
   }

   @Override
   protected void setDefaultConstraints() {
       super.setDefaultConstraints();
       // Set default contraints
       m_errorConsole.setConstraint(new GConstraint(GConstraint.push(), GConstraint.grow()));
       m_errorConsolePan.setConstraint(new GConstraint(GConstraint.newline(), GConstraint.spanx(2),
               GConstraint.pushx(), GConstraint.growx()));
       m_errorConsolePan.setMargins(0, 0, 0, 0);

       // Link with data panel
       m_dataPanel.setConstraint(new GConstraint(GConstraint.newline(), GConstraint.push(),
               GConstraint.grow(), GConstraint.height(dataPanelHeight)));
       m_dataPanelPan.setConstraint(new GConstraint(GConstraint.newline(), GConstraint.spanx(2),
               GConstraint.push(), GConstraint.grow(), GConstraint.height(dataPanelHeight)));
       
       m_split.getJSplitPane().setDividerLocation(errConsoleHeight);
       m_split.setConstraint(new GConstraint(GConstraint.newline(), GConstraint.spanx(2),
                   GConstraint.grow(), GConstraint.push()));


       if (m_progressBar != null) {
           m_progressBar.setConstraint(new GConstraint(GConstraint.newline(false), GConstraint
                   .growx(), GConstraint.gapleft(GAP_LEFT), GConstraint.gapright(GAP_RIGHT)));
       }
       contextLabel.setConstraint(new GConstraint(GConstraint.newline(), GConstraint.split(2),
               GConstraint.spanx(2)));
       contextFileNameLabel.setConstraint(new GConstraint(GConstraint.newline(false), GConstraint
               .gapleft(GAP_LEFT)));

   }

   /**
    * Inisialise the static strings.
    */
   private void initStrings() {
       if (sDebugFalseString == null) {
           sDebugFalseString = GEnvironment.tr("Change Debug Mode (currently No)");
           sDebugTrueString = GEnvironment.tr("Change Debug Mode (currently Yes)");

           s_loadContextString = GEnvironment.tr("Load context...");
           s_saveContextString = GEnvironment.tr("Save context...");
           s_saveResultsString = GEnvironment.tr("Save results...");
           s_clearMessages = GEnvironment.tr("Clear messages");
           s_clearResults = GEnvironment.tr("Clear results");
           s_quitString = GEnvironment.tr("Quit");
       }
   }

   /**
    * Create the menu bar.
    */
   private void createMenuBar() {

       // Items of File Menu initialization
       m_menuItemLoadContext = new GMenuItem(s_loadContextString);
       m_menuItemLoadContext.setShortcut(KeyEvent.VK_L);
       m_menuItemLoadContext.setAccelerator(KeyEvent.VK_L, ActionEvent.CTRL_MASK);

       m_menuItemSaveContext = new GMenuItem(s_saveContextString);
       m_menuItemSaveContext.setShortcut(KeyEvent.VK_S);
       m_menuItemSaveContext.setAccelerator(KeyEvent.VK_S, ActionEvent.CTRL_MASK);

       m_menuItemSaveResults = new GMenuItem(s_saveResultsString);
       m_menuItemSaveResults.setShortcut(KeyEvent.VK_R);
       m_menuItemSaveResults.setAccelerator(KeyEvent.VK_R, ActionEvent.CTRL_MASK);

       m_menuItemRun = m_javaCommandLauncher.getGExecMenuItem();
       m_menuItemRun.setShortcut(KeyEvent.VK_T);
       m_menuItemRun.setAccelerator(KeyEvent.VK_T, ActionEvent.CTRL_MASK);

       m_menuItemQuit = new GMenuItem(s_quitString);

       // Items of Windows menu initialization
       m_menuItemClearResults = new GMenuItem(s_clearResults);
       m_menuItemClearMessages = new GMenuItem(s_clearMessages);

       // Items of Options Menu initialization
       m_menuItemDebugMode = new GMenuItem(GEnvironment.tr(sDebugFalseString));

       // Items of About menu initialization
       m_menuItemAbout = new GMenuItem(GEnvironment.tr("?"));

       // FileMenu initialization
       final GMenu fileMenu = new GMenu(GEnvironment.tr("File"));
       fileMenu.add(m_menuItemLoadContext);
       fileMenu.add(m_menuItemSaveContext);
       fileMenu.add(m_menuItemSaveResults);
       fileMenu.add(m_menuItemRun);
       fileMenu.add(m_menuItemQuit);
       fileMenu.setShortcut(KeyEvent.VK_M);

       // WindowsMenu initialization
       final GMenu windowsMenu = new GMenu(GEnvironment.tr("Windows"));
       windowsMenu.add(m_menuItemClearResults);
       windowsMenu.add(m_menuItemClearMessages);

       // OptionMenu initialization
       final GMenu optionMenu = new GMenu(GEnvironment.tr("Options"));
       optionMenu.add(m_menuItemDebugMode);

       // AboutMenu initialization
       final GMenu aboutMenu = new GMenu(GEnvironment.tr("About"));
       aboutMenu.add(m_menuItemAbout);

       // Menu Bar initialization
       m_menuBar = new GMenuBar(this);
       m_menuBar.add(fileMenu);
       m_menuBar.add(windowsMenu);
       m_menuBar.add(optionMenu);
       m_menuBar.add(aboutMenu);

   }

   /**
    * Create the button bar.
    * 
    * @param iconSize size of the icons in pixels.
    */
   private void createButtonBar(final int iconSize) {

       int numberButtons = 6;
       if (m_progressBar != null) {
           numberButtons++;
       }

       // Icons initialization
       m_loadContextButton = new GButton(s_loadContextString);
       m_loadContextButton.setIcon(new GIcon(GIcon.Type.OPEN, iconSize));
       m_loadContextButton.setConstraint(new GConstraint(GConstraint.split(numberButtons)));

       m_saveContextButton = new GButton(s_saveContextString);
       m_saveContextButton.setIcon(new GIcon(GIcon.Type.SAVE, iconSize));
       m_saveContextButton.setConstraint(null);

       m_saveResultsButton = new GButton(s_saveResultsString);
       m_saveResultsButton.setIcon(new GIcon(GIcon.Type.SAVE, iconSize));
       m_saveResultsButton.setConstraint(null);

       m_runButton = m_javaCommandLauncher.getGExecButton();
       m_runButton.setConstraint(null);

       m_clearResultsButton = new GButton(s_clearResults);
       m_clearResultsButton.setConstraint(null);
       m_clearResultsButton.setIcon(new GIcon(GIcon.Type.CLEAR, iconSize));

       m_clearMessagesButton = new GButton(s_clearMessages);
       m_clearMessagesButton.setConstraint(null);
       m_clearMessagesButton.setIcon(new GIcon(GIcon.Type.CLEAR, iconSize));

       m_quitButton = new GButton(s_quitString);
       m_quitButton.setConstraint(new GConstraint(GConstraint.newline(false), GConstraint
               .alignx(XAlign.Right), GConstraint.pushx()));
       m_quitButton.setIcon(new GIcon(GIcon.Type.EXIT, iconSize));
   }

   /**
    * Get the data panel instance.
    * 
    * @return the data panel instance.
    */
   public T getDataPanel() {
       return m_dataPanel;
   }

   /**
    * Get the save results dialog instance.
    * 
    * @return the save results dialog instance.
    */
   public GSaveResults getSaveResultsDialog() {
       return m_saveResultsDialog;
   }

   /**
    * Get the command launcher instance.
    * 
    * @return the command launcher instance.
    */
   public GJavaCommandLauncher getJavaCommandLauncher() {
       return m_javaCommandLauncher;
   }

   /**
    * Display the main frame.
    */
   public void displayMainFrame() {
       m_mainFrame.display();
   }

   /**
    * Set the minimum size of the main frame.
    * 
    * @param width minimum width.
    * @param height minimum height.
    */
   public void setMainFrameMinimumSize(final int width, final int height) {
       m_mainFrame.setMinimumSize(new Dimension(width, height));
   }

   /**
    * {@inheritDoc}
    */
   @Override
   public void display() throws GException {
       generic();
   }

   /**
    * {@inheritDoc}
    */
   @Override
   public void generic() throws GException {
       put(m_loadContextButton);
       put(m_saveContextButton);
       put(m_saveResultsButton);
       put(m_runButton);
       put(m_clearResultsButton);
       put(m_clearMessagesButton);

       if (m_progressBar != null) {
           put(m_progressBar);
       }
       put(m_quitButton);
//       put(m_errorConsole);
       put(m_split);
//       put(contextLabel);
//       put(contextFileNameLabel);
//       put(m_dataPanel);
       put(m_javaCommandLauncher);
   }

   /**
    * {@inheritDoc}
    */
   public void before(final GEvent event) throws GException {

       // Launch the compuitation
       if (event.contains(m_menuItemRun, m_runButton)) {
           onBeforeRun();
       }
   }

   /**
    * Steps to run before running the processus.
    */
   private void onBeforeRun() {
       if (!m_javaCommandLauncher.isRunning()) {

           if (m_dataPanel.getStatus() == GStatus.ERROR) {

               GConsoleLogger.getLogger().log(Level.SEVERE, "there are some erroneous data ...");
               m_javaCommandLauncher.setInhibited(true);

           } else {

               if (m_dataPanel.getStatus() == GStatus.WARN) {
                   GConsoleLogger.getLogger().log(Level.WARNING, "some data raised warning ...");
               }

               // Custom preprocessing management
               try {
                   customPreProcessManagement();
                   //JFG
//                   if ( m_javaCommandLauncher.getInhibited() ) {
//                       GConsoleLogger.getLogger().log(Level.INFO, "start computation aborted ...");
//                   } else {
                       GConsoleLogger.getLogger().log(Level.INFO, "start computation ...");
//                   }
                   //JFG
               } catch (GFileManipulatorException e) {
                   GConsoleLogger.getLogger().log(Level.INFO, "start computation failed ...");
               }

           }
       }

   }

   /**
    * Method managing what occurs after action.
    * 
    * @param eve GENIUS event.
    * @throws GException GENIUS exception
    */
   @SuppressWarnings("PMD.ModifiedCyclomaticComplexity")
   public void after(final GEvent eve) throws GException {

       if (eve.contains(m_menuItemLoadContext, m_loadContextButton)) {
           contextFileName = m_contextFileManagement.selectLoadFile(m_dataPanel, true);
           contextFileNameLabel.setText(contextFileName);
       }

       if (eve.contains(m_menuItemSaveContext, m_saveContextButton)) {
           contextFileName = m_contextFileManagement.selectSaveFile(m_dataPanel, true);
           contextFileNameLabel.setText(contextFileName);
       }

       if (eve.contains(m_menuItemSaveResults, m_saveResultsButton)) {
           saveFilesManagement();
           contextFileNameLabel.setText(contextFileName);
       }

       if (eve.contains(m_menuItemQuit, m_quitButton)) {
           m_mainFrame.dispose();
       }

       if (eve.contains(m_menuItemClearResults, m_clearResultsButton)) {
           m_dataPanel.getConsole().reset();
       }

       if (eve.contains(m_menuItemClearMessages, m_clearMessagesButton)) {
           m_errorConsole.reset();
       }

       if (eve.contains(m_menuItemDebugMode)) {
           toggleDebugMode();

           if (sDebugMode) {
               m_menuItemDebugMode.setText(sDebugTrueString);
           } else {
               m_menuItemDebugMode.setText(sDebugFalseString);
           }
       }

       if (eve.contains(m_menuItemAbout)) {
           m_aboutDialog.show();
       }

       if (eve.contains(m_dataPanel)) {
           contextFileNameLabel.setText(contextFileName + " (*)");
       }

       // Application started ...
       if (eve.getFinalSource() == m_javaCommandLauncher) {
           if (m_javaCommandLauncher.getProcessStatus() == ProcessStatus.FINISHED_NORMALY) {
               GConsoleLogger.getLogger().log(Level.INFO, "end of computation ...");
           } else if (m_javaCommandLauncher.getProcessStatus() == ProcessStatus.FINISHED_BY_USER) {
               GConsoleLogger.getLogger().log(Level.WARNING, "computation stopped by user ...");
           }
           customPostProcessManagement();
       }

   }

   /**
    * Method allowing to set font of the error console.
    * 
    * @param errConsoleFont font used for output error console.
    */
   public void setErrConsoleFont(final Font errConsoleFont) {
       m_errorConsole.getJTextArea().setFont(errConsoleFont);
   }

   /**
    * Method to return the menu bar to be called by the main static method.
    * 
    * @return menu bar
    */
   public GMenuBar getMenuBar() {
       return m_menuBar;
   }

   /**
    * Abstract method where setting mechanism for result (and context) files saving.
    * 
    * @throws GException GENIUS exception
    */
   protected abstract void saveFilesManagement() throws GException;

   /**
    * Abstract method where setting customized pre process management.
    * 
    * @throws GFileManipulatorException exception when error after reading context files.
    */
   protected abstract void customPreProcessManagement() throws GFileManipulatorException;

   /**
    * Abstract method where setting customized post process management.
    */
   protected abstract void customPostProcessManagement();

   /**
    * Method to get debug mode.
    * 
    * @return the debug mode.
    */
   public static boolean getDebugMode() {
       return sDebugMode;
   }

   /**
    * Method to configure in debug mode or not.
    * 
    * @param mode debug mode if true
    */
   public static void setDebugMode(final boolean mode) {
       sDebugMode = mode;
   }

   /**
    * Changes the debug mode value by its opposite.
    */
   private static final void toggleDebugMode() {
       sDebugMode = !sDebugMode;
   }

   /**
    * Static method to get the default font size.
    * 
    * @return default font size
    */
   private static int getDefaultFontSize() {
       final Font tmp = UIManager.getDefaults().getFont("TextField.font");
       return tmp.getSize();
   }

}
